@@ -16,6 +16,7 @@ __PACKAGE__->add_columns(
 __PACKAGE__->set_primary_key("dvd", "tag");
 __PACKAGE__->belongs_to("dvd", "DBSchema::Result::Dvd", { dvd_id => "dvd" });
 __PACKAGE__->belongs_to("tag", "DBSchema::Result::Tag", { id => "tag" });
+# same relationship with a name that's different from the column name
+__PACKAGE__->belongs_to("rel_tag", "DBSchema::Result::Tag", { id => "tag" });
 
 1;
-
